# coding: utf-8

# flake8: noqa
"""
    ACROSS Server

    Server providing tools and utilities for various NASA missions to aid in coordination of large observation efforts.

    The version of the OpenAPI document: 1.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from across.sdk.v1.models.access_token_response import AccessTokenResponse
from across.sdk.v1.models.across_server_routes_v1_group_invite_schemas_group_invite import AcrossServerRoutesV1GroupInviteSchemasGroupInvite
from across.sdk.v1.models.across_server_routes_v1_group_role_schemas_group_role import AcrossServerRoutesV1GroupRoleSchemasGroupRole
from across.sdk.v1.models.across_server_routes_v1_group_role_schemas_service_account import AcrossServerRoutesV1GroupRoleSchemasServiceAccount
from across.sdk.v1.models.across_server_routes_v1_group_role_schemas_user import AcrossServerRoutesV1GroupRoleSchemasUser
from across.sdk.v1.models.across_server_routes_v1_group_schemas_group import AcrossServerRoutesV1GroupSchemasGroup
from across.sdk.v1.models.across_server_routes_v1_group_schemas_user import AcrossServerRoutesV1GroupSchemasUser
from across.sdk.v1.models.across_server_routes_v1_user_schemas_group import AcrossServerRoutesV1UserSchemasGroup
from across.sdk.v1.models.across_server_routes_v1_user_schemas_group_invite import AcrossServerRoutesV1UserSchemasGroupInvite
from across.sdk.v1.models.across_server_routes_v1_user_schemas_group_role import AcrossServerRoutesV1UserSchemasGroupRole
from across.sdk.v1.models.across_server_routes_v1_user_schemas_user import AcrossServerRoutesV1UserSchemasUser
from across.sdk.v1.models.across_server_routes_v1_user_service_account_schemas_service_account import AcrossServerRoutesV1UserServiceAccountSchemasServiceAccount
from across.sdk.v1.models.alt_az_constraint import AltAzConstraint
from across.sdk.v1.models.bandpass import Bandpass
from across.sdk.v1.models.bandpass_type import BandpassType
from across.sdk.v1.models.constrained_date import ConstrainedDate
from across.sdk.v1.models.constraint_reason import ConstraintReason
from across.sdk.v1.models.constraint_type import ConstraintType
from across.sdk.v1.models.coordinate import Coordinate
from across.sdk.v1.models.date_range import DateRange
from across.sdk.v1.models.depth_unit import DepthUnit
from across.sdk.v1.models.earth_limb_constraint import EarthLimbConstraint
from across.sdk.v1.models.energy_bandpass import EnergyBandpass
from across.sdk.v1.models.energy_unit import EnergyUnit
from across.sdk.v1.models.ephemeris_type import EphemerisType
from across.sdk.v1.models.filter import Filter
from across.sdk.v1.models.frequency_bandpass import FrequencyBandpass
from across.sdk.v1.models.frequency_unit import FrequencyUnit
from across.sdk.v1.models.grant_type import GrantType
from across.sdk.v1.models.ground_parameters import GroundParameters
from across.sdk.v1.models.group_invite_create import GroupInviteCreate
from across.sdk.v1.models.group_invite_group_details import GroupInviteGroupDetails
from across.sdk.v1.models.group_read import GroupRead
from across.sdk.v1.models.group_role_create import GroupRoleCreate
from across.sdk.v1.models.group_role_read import GroupRoleRead
from across.sdk.v1.models.http_validation_error import HTTPValidationError
from across.sdk.v1.models.id_name_schema import IDNameSchema
from across.sdk.v1.models.ivoa_obs_category import IVOAObsCategory
from across.sdk.v1.models.ivoa_obs_tracking_type import IVOAObsTrackingType
from across.sdk.v1.models.instrument import Instrument
from across.sdk.v1.models.instrument_constraints_inner import InstrumentConstraintsInner
from across.sdk.v1.models.jpl_parameters import JPLParameters
from across.sdk.v1.models.joint_visibility_result import JointVisibilityResult
from across.sdk.v1.models.moon_angle_constraint import MoonAngleConstraint
from across.sdk.v1.models.nullable_date_range import NullableDateRange
from across.sdk.v1.models.observation import Observation
from across.sdk.v1.models.observation_create import ObservationCreate
from across.sdk.v1.models.observation_status import ObservationStatus
from across.sdk.v1.models.observation_type import ObservationType
from across.sdk.v1.models.observatory import Observatory
from across.sdk.v1.models.observatory_ephemeris_type import ObservatoryEphemerisType
from across.sdk.v1.models.observatory_type import ObservatoryType
from across.sdk.v1.models.page_observation import PageObservation
from across.sdk.v1.models.page_schedule import PageSchedule
from across.sdk.v1.models.parameters import Parameters
from across.sdk.v1.models.permission import Permission
from across.sdk.v1.models.point import Point
from across.sdk.v1.models.role_base import RoleBase
from across.sdk.v1.models.saa_polygon_constraint import SAAPolygonConstraint
from across.sdk.v1.models.spice_parameters import SPICEParameters
from across.sdk.v1.models.schedule import Schedule
from across.sdk.v1.models.schedule_cadence import ScheduleCadence
from across.sdk.v1.models.schedule_create import ScheduleCreate
from across.sdk.v1.models.schedule_create_many import ScheduleCreateMany
from across.sdk.v1.models.schedule_fidelity import ScheduleFidelity
from across.sdk.v1.models.schedule_status import ScheduleStatus
from across.sdk.v1.models.service_account_create import ServiceAccountCreate
from across.sdk.v1.models.service_account_secret import ServiceAccountSecret
from across.sdk.v1.models.service_account_update import ServiceAccountUpdate
from across.sdk.v1.models.sun_angle_constraint import SunAngleConstraint
from across.sdk.v1.models.system_service_account import SystemServiceAccount
from across.sdk.v1.models.tle import TLE
from across.sdk.v1.models.tle_create import TLECreate
from across.sdk.v1.models.tle_parameters import TLEParameters
from across.sdk.v1.models.telescope import Telescope
from across.sdk.v1.models.telescope_instrument import TelescopeInstrument
from across.sdk.v1.models.unit_value import UnitValue
from across.sdk.v1.models.user_create import UserCreate
from across.sdk.v1.models.user_info import UserInfo
from across.sdk.v1.models.user_update import UserUpdate
from across.sdk.v1.models.validation_error import ValidationError
from across.sdk.v1.models.validation_error_loc_inner import ValidationErrorLocInner
from across.sdk.v1.models.visibility_result import VisibilityResult
from across.sdk.v1.models.visibility_type import VisibilityType
from across.sdk.v1.models.visibility_window import VisibilityWindow
from across.sdk.v1.models.wavelength_bandpass import WavelengthBandpass
from across.sdk.v1.models.wavelength_unit import WavelengthUnit
from across.sdk.v1.models.window import Window



# coding: utf-8

"""
    ACROSS Server

    Server providing tools and utilities for various NASA missions to aid in coordination of large observation efforts.

    The version of the OpenAPI document: 1.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from datetime import datetime
from pydantic import Field, StrictFloat, StrictInt, StrictStr
from typing import Any, List, Optional, Union
from typing_extensions import Annotated
from across.sdk.v1.models.depth_unit import DepthUnit
from across.sdk.v1.models.observation import Observation
from across.sdk.v1.models.observation_status import ObservationStatus
from across.sdk.v1.models.observation_type import ObservationType
from across.sdk.v1.models.page_observation import PageObservation

from across.sdk.v1.api_client import ApiClient, RequestSerialized
from across.sdk.v1.api_response import ApiResponse
from across.sdk.v1.rest import RESTResponseType


class ObservationApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def get_observation(
        self,
        observation_id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Observation:
        """Read an observation

        Read an observation by an observation ID.

        :param observation_id: (required)
        :type observation_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_observation_serialize(
            observation_id=observation_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Observation",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_observation_with_http_info(
        self,
        observation_id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Observation]:
        """Read an observation

        Read an observation by an observation ID.

        :param observation_id: (required)
        :type observation_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_observation_serialize(
            observation_id=observation_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Observation",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_observation_without_preload_content(
        self,
        observation_id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Read an observation

        Read an observation by an observation ID.

        :param observation_id: (required)
        :type observation_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_observation_serialize(
            observation_id=observation_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Observation",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_observation_serialize(
        self,
        observation_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if observation_id is not None:
            _path_params['observation_id'] = observation_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/observation/{observation_id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_observations(
        self,
        page: Annotated[Optional[Annotated[int, Field(strict=True, ge=1)]], Field(description="Page number")] = None,
        page_limit: Annotated[Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]], Field(description="Records per page")] = None,
        external_id: Optional[StrictStr] = None,
        schedule_ids: Optional[List[Optional[StrictStr]]] = None,
        observatory_ids: Optional[List[StrictStr]] = None,
        telescope_ids: Optional[List[StrictStr]] = None,
        instrument_ids: Optional[List[StrictStr]] = None,
        status: Optional[ObservationStatus] = None,
        proposal: Optional[StrictStr] = None,
        object_name: Optional[StrictStr] = None,
        date_range_begin: Optional[datetime] = None,
        date_range_end: Optional[datetime] = None,
        bandpass_min: Optional[Union[StrictFloat, StrictInt]] = None,
        bandpass_max: Optional[Union[StrictFloat, StrictInt]] = None,
        bandpass_type: Optional[Any] = None,
        cone_search_ra: Optional[Union[StrictFloat, StrictInt]] = None,
        cone_search_dec: Optional[Union[StrictFloat, StrictInt]] = None,
        cone_search_radius: Optional[Union[StrictFloat, StrictInt]] = None,
        type: Optional[ObservationType] = None,
        depth_value: Optional[Union[StrictFloat, StrictInt]] = None,
        depth_unit: Optional[DepthUnit] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PageObservation:
        """Read observations(s)

        Read the observations based on query params

        :param page: Page number
        :type page: int
        :param page_limit: Records per page
        :type page_limit: int
        :param external_id:
        :type external_id: str
        :param schedule_ids:
        :type schedule_ids: List[Optional[str]]
        :param observatory_ids:
        :type observatory_ids: List[str]
        :param telescope_ids:
        :type telescope_ids: List[str]
        :param instrument_ids:
        :type instrument_ids: List[str]
        :param status:
        :type status: ObservationStatus
        :param proposal:
        :type proposal: str
        :param object_name:
        :type object_name: str
        :param date_range_begin:
        :type date_range_begin: datetime
        :param date_range_end:
        :type date_range_end: datetime
        :param bandpass_min:
        :type bandpass_min: float
        :param bandpass_max:
        :type bandpass_max: float
        :param bandpass_type:
        :type bandpass_type: BandpassType
        :param cone_search_ra:
        :type cone_search_ra: float
        :param cone_search_dec:
        :type cone_search_dec: float
        :param cone_search_radius:
        :type cone_search_radius: float
        :param type:
        :type type: ObservationType
        :param depth_value:
        :type depth_value: float
        :param depth_unit:
        :type depth_unit: DepthUnit
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_observations_serialize(
            page=page,
            page_limit=page_limit,
            external_id=external_id,
            schedule_ids=schedule_ids,
            observatory_ids=observatory_ids,
            telescope_ids=telescope_ids,
            instrument_ids=instrument_ids,
            status=status,
            proposal=proposal,
            object_name=object_name,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            bandpass_min=bandpass_min,
            bandpass_max=bandpass_max,
            bandpass_type=bandpass_type,
            cone_search_ra=cone_search_ra,
            cone_search_dec=cone_search_dec,
            cone_search_radius=cone_search_radius,
            type=type,
            depth_value=depth_value,
            depth_unit=depth_unit,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PageObservation",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_observations_with_http_info(
        self,
        page: Annotated[Optional[Annotated[int, Field(strict=True, ge=1)]], Field(description="Page number")] = None,
        page_limit: Annotated[Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]], Field(description="Records per page")] = None,
        external_id: Optional[StrictStr] = None,
        schedule_ids: Optional[List[Optional[StrictStr]]] = None,
        observatory_ids: Optional[List[StrictStr]] = None,
        telescope_ids: Optional[List[StrictStr]] = None,
        instrument_ids: Optional[List[StrictStr]] = None,
        status: Optional[ObservationStatus] = None,
        proposal: Optional[StrictStr] = None,
        object_name: Optional[StrictStr] = None,
        date_range_begin: Optional[datetime] = None,
        date_range_end: Optional[datetime] = None,
        bandpass_min: Optional[Union[StrictFloat, StrictInt]] = None,
        bandpass_max: Optional[Union[StrictFloat, StrictInt]] = None,
        bandpass_type: Optional[Any] = None,
        cone_search_ra: Optional[Union[StrictFloat, StrictInt]] = None,
        cone_search_dec: Optional[Union[StrictFloat, StrictInt]] = None,
        cone_search_radius: Optional[Union[StrictFloat, StrictInt]] = None,
        type: Optional[ObservationType] = None,
        depth_value: Optional[Union[StrictFloat, StrictInt]] = None,
        depth_unit: Optional[DepthUnit] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PageObservation]:
        """Read observations(s)

        Read the observations based on query params

        :param page: Page number
        :type page: int
        :param page_limit: Records per page
        :type page_limit: int
        :param external_id:
        :type external_id: str
        :param schedule_ids:
        :type schedule_ids: List[Optional[str]]
        :param observatory_ids:
        :type observatory_ids: List[str]
        :param telescope_ids:
        :type telescope_ids: List[str]
        :param instrument_ids:
        :type instrument_ids: List[str]
        :param status:
        :type status: ObservationStatus
        :param proposal:
        :type proposal: str
        :param object_name:
        :type object_name: str
        :param date_range_begin:
        :type date_range_begin: datetime
        :param date_range_end:
        :type date_range_end: datetime
        :param bandpass_min:
        :type bandpass_min: float
        :param bandpass_max:
        :type bandpass_max: float
        :param bandpass_type:
        :type bandpass_type: BandpassType
        :param cone_search_ra:
        :type cone_search_ra: float
        :param cone_search_dec:
        :type cone_search_dec: float
        :param cone_search_radius:
        :type cone_search_radius: float
        :param type:
        :type type: ObservationType
        :param depth_value:
        :type depth_value: float
        :param depth_unit:
        :type depth_unit: DepthUnit
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_observations_serialize(
            page=page,
            page_limit=page_limit,
            external_id=external_id,
            schedule_ids=schedule_ids,
            observatory_ids=observatory_ids,
            telescope_ids=telescope_ids,
            instrument_ids=instrument_ids,
            status=status,
            proposal=proposal,
            object_name=object_name,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            bandpass_min=bandpass_min,
            bandpass_max=bandpass_max,
            bandpass_type=bandpass_type,
            cone_search_ra=cone_search_ra,
            cone_search_dec=cone_search_dec,
            cone_search_radius=cone_search_radius,
            type=type,
            depth_value=depth_value,
            depth_unit=depth_unit,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PageObservation",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_observations_without_preload_content(
        self,
        page: Annotated[Optional[Annotated[int, Field(strict=True, ge=1)]], Field(description="Page number")] = None,
        page_limit: Annotated[Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]], Field(description="Records per page")] = None,
        external_id: Optional[StrictStr] = None,
        schedule_ids: Optional[List[Optional[StrictStr]]] = None,
        observatory_ids: Optional[List[StrictStr]] = None,
        telescope_ids: Optional[List[StrictStr]] = None,
        instrument_ids: Optional[List[StrictStr]] = None,
        status: Optional[ObservationStatus] = None,
        proposal: Optional[StrictStr] = None,
        object_name: Optional[StrictStr] = None,
        date_range_begin: Optional[datetime] = None,
        date_range_end: Optional[datetime] = None,
        bandpass_min: Optional[Union[StrictFloat, StrictInt]] = None,
        bandpass_max: Optional[Union[StrictFloat, StrictInt]] = None,
        bandpass_type: Optional[Any] = None,
        cone_search_ra: Optional[Union[StrictFloat, StrictInt]] = None,
        cone_search_dec: Optional[Union[StrictFloat, StrictInt]] = None,
        cone_search_radius: Optional[Union[StrictFloat, StrictInt]] = None,
        type: Optional[ObservationType] = None,
        depth_value: Optional[Union[StrictFloat, StrictInt]] = None,
        depth_unit: Optional[DepthUnit] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Read observations(s)

        Read the observations based on query params

        :param page: Page number
        :type page: int
        :param page_limit: Records per page
        :type page_limit: int
        :param external_id:
        :type external_id: str
        :param schedule_ids:
        :type schedule_ids: List[Optional[str]]
        :param observatory_ids:
        :type observatory_ids: List[str]
        :param telescope_ids:
        :type telescope_ids: List[str]
        :param instrument_ids:
        :type instrument_ids: List[str]
        :param status:
        :type status: ObservationStatus
        :param proposal:
        :type proposal: str
        :param object_name:
        :type object_name: str
        :param date_range_begin:
        :type date_range_begin: datetime
        :param date_range_end:
        :type date_range_end: datetime
        :param bandpass_min:
        :type bandpass_min: float
        :param bandpass_max:
        :type bandpass_max: float
        :param bandpass_type:
        :type bandpass_type: BandpassType
        :param cone_search_ra:
        :type cone_search_ra: float
        :param cone_search_dec:
        :type cone_search_dec: float
        :param cone_search_radius:
        :type cone_search_radius: float
        :param type:
        :type type: ObservationType
        :param depth_value:
        :type depth_value: float
        :param depth_unit:
        :type depth_unit: DepthUnit
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_observations_serialize(
            page=page,
            page_limit=page_limit,
            external_id=external_id,
            schedule_ids=schedule_ids,
            observatory_ids=observatory_ids,
            telescope_ids=telescope_ids,
            instrument_ids=instrument_ids,
            status=status,
            proposal=proposal,
            object_name=object_name,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            bandpass_min=bandpass_min,
            bandpass_max=bandpass_max,
            bandpass_type=bandpass_type,
            cone_search_ra=cone_search_ra,
            cone_search_dec=cone_search_dec,
            cone_search_radius=cone_search_radius,
            type=type,
            depth_value=depth_value,
            depth_unit=depth_unit,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PageObservation",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_observations_serialize(
        self,
        page,
        page_limit,
        external_id,
        schedule_ids,
        observatory_ids,
        telescope_ids,
        instrument_ids,
        status,
        proposal,
        object_name,
        date_range_begin,
        date_range_end,
        bandpass_min,
        bandpass_max,
        bandpass_type,
        cone_search_ra,
        cone_search_dec,
        cone_search_radius,
        type,
        depth_value,
        depth_unit,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'schedule_ids': 'multi',
            'observatory_ids': 'multi',
            'telescope_ids': 'multi',
            'instrument_ids': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if page is not None:
            
            _query_params.append(('page', page))
            
        if page_limit is not None:
            
            _query_params.append(('page_limit', page_limit))
            
        if external_id is not None:
            
            _query_params.append(('external_id', external_id))
            
        if schedule_ids is not None:
            
            _query_params.append(('schedule_ids', schedule_ids))
            
        if observatory_ids is not None:
            
            _query_params.append(('observatory_ids', observatory_ids))
            
        if telescope_ids is not None:
            
            _query_params.append(('telescope_ids', telescope_ids))
            
        if instrument_ids is not None:
            
            _query_params.append(('instrument_ids', instrument_ids))
            
        if status is not None:
            
            _query_params.append(('status', status.value))
            
        if proposal is not None:
            
            _query_params.append(('proposal', proposal))
            
        if object_name is not None:
            
            _query_params.append(('object_name', object_name))
            
        if date_range_begin is not None:
            if isinstance(date_range_begin, datetime):
                _query_params.append(
                    (
                        'date_range_begin',
                        date_range_begin.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('date_range_begin', date_range_begin))
            
        if date_range_end is not None:
            if isinstance(date_range_end, datetime):
                _query_params.append(
                    (
                        'date_range_end',
                        date_range_end.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('date_range_end', date_range_end))
            
        if bandpass_min is not None:
            
            _query_params.append(('bandpass_min', bandpass_min))
            
        if bandpass_max is not None:
            
            _query_params.append(('bandpass_max', bandpass_max))
            
        if bandpass_type is not None:
            
            _query_params.append(('bandpass_type', bandpass_type))
            
        if cone_search_ra is not None:
            
            _query_params.append(('cone_search_ra', cone_search_ra))
            
        if cone_search_dec is not None:
            
            _query_params.append(('cone_search_dec', cone_search_dec))
            
        if cone_search_radius is not None:
            
            _query_params.append(('cone_search_radius', cone_search_radius))
            
        if type is not None:
            
            _query_params.append(('type', type.value))
            
        if depth_value is not None:
            
            _query_params.append(('depth_value', depth_value))
            
        if depth_unit is not None:
            
            _query_params.append(('depth_unit', depth_unit.value))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/observation/',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

"""
    ACROSS Server

    Server providing tools and utilities for various NASA missions to aid in coordination of large observation efforts.

    The version of the OpenAPI document: 1.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class WavelengthUnit(str, Enum):
    """
    Enum to represent the bandpass wavelength
    """

    """
    allowed enum values
    """
    NM = 'nm'
    ANGSTROM = 'angstrom'
    UM = 'um'
    MM = 'mm'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of WavelengthUnit from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    ACROSS Server

    Server providing tools and utilities for various NASA missions to aid in coordination of large observation efforts.

    The version of the OpenAPI document: 1.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List
from across.sdk.v1.models.across_server_routes_v1_user_schemas_group import AcrossServerRoutesV1UserSchemasGroup
from across.sdk.v1.models.across_server_routes_v1_user_schemas_group_invite import AcrossServerRoutesV1UserSchemasGroupInvite
from across.sdk.v1.models.across_server_routes_v1_user_schemas_group_role import AcrossServerRoutesV1UserSchemasGroupRole
from across.sdk.v1.models.role_base import RoleBase
from typing import Optional, Set
from typing_extensions import Self

class AcrossServerRoutesV1UserSchemasUser(BaseModel):
    """
    AcrossServerRoutesV1UserSchemasUser
    """ # noqa: E501
    username: StrictStr
    first_name: StrictStr
    last_name: StrictStr
    email: StrictStr
    id: StrictStr
    groups: List[AcrossServerRoutesV1UserSchemasGroup]
    roles: List[RoleBase]
    group_roles: List[AcrossServerRoutesV1UserSchemasGroupRole]
    received_invites: List[AcrossServerRoutesV1UserSchemasGroupInvite]
    __properties: ClassVar[List[str]] = ["username", "first_name", "last_name", "email", "id", "groups", "roles", "group_roles", "received_invites"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of AcrossServerRoutesV1UserSchemasUser from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in groups (list)
        _items = []
        if self.groups:
            for _item_groups in self.groups:
                if _item_groups:
                    _items.append(_item_groups.to_dict())
            _dict['groups'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in roles (list)
        _items = []
        if self.roles:
            for _item_roles in self.roles:
                if _item_roles:
                    _items.append(_item_roles.to_dict())
            _dict['roles'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in group_roles (list)
        _items = []
        if self.group_roles:
            for _item_group_roles in self.group_roles:
                if _item_group_roles:
                    _items.append(_item_group_roles.to_dict())
            _dict['group_roles'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in received_invites (list)
        _items = []
        if self.received_invites:
            for _item_received_invites in self.received_invites:
                if _item_received_invites:
                    _items.append(_item_received_invites.to_dict())
            _dict['received_invites'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AcrossServerRoutesV1UserSchemasUser from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "username": obj.get("username"),
            "first_name": obj.get("first_name"),
            "last_name": obj.get("last_name"),
            "email": obj.get("email"),
            "id": obj.get("id"),
            "groups": [AcrossServerRoutesV1UserSchemasGroup.from_dict(_item) for _item in obj["groups"]] if obj.get("groups") is not None else None,
            "roles": [RoleBase.from_dict(_item) for _item in obj["roles"]] if obj.get("roles") is not None else None,
            "group_roles": [AcrossServerRoutesV1UserSchemasGroupRole.from_dict(_item) for _item in obj["group_roles"]] if obj.get("group_roles") is not None else None,
            "received_invites": [AcrossServerRoutesV1UserSchemasGroupInvite.from_dict(_item) for _item in obj["received_invites"]] if obj.get("received_invites") is not None else None
        })
        return _obj





# coding: utf-8

"""
    ACROSS Server

    Server providing tools and utilities for various NASA missions to aid in coordination of large observation efforts.

    The version of the OpenAPI document: 1.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing import Optional, Set
from typing_extensions import Self

class Filter(BaseModel):
    """
    A Pydantic model class representing a created Filter  Notes ----- Inherits from FilterBase
    """ # noqa: E501
    id: StrictStr
    created_on: datetime
    name: StrictStr
    peak_wavelength: Optional[Union[StrictFloat, StrictInt]]
    min_wavelength: Union[StrictFloat, StrictInt]
    max_wavelength: Union[StrictFloat, StrictInt]
    is_operational: StrictBool
    sensitivity_depth_unit: Optional[Union[StrictFloat, StrictInt]]
    sensitivity_depth: Optional[Union[StrictFloat, StrictInt]]
    sensitivity_time_seconds: Optional[Union[StrictFloat, StrictInt]]
    reference_url: Optional[StrictStr]
    instrument_id: StrictStr
    __properties: ClassVar[List[str]] = ["id", "created_on", "name", "peak_wavelength", "min_wavelength", "max_wavelength", "is_operational", "sensitivity_depth_unit", "sensitivity_depth", "sensitivity_time_seconds", "reference_url", "instrument_id"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Filter from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if peak_wavelength (nullable) is None
        # and model_fields_set contains the field
        if self.peak_wavelength is None and "peak_wavelength" in self.model_fields_set:
            _dict['peak_wavelength'] = None

        # set to None if sensitivity_depth_unit (nullable) is None
        # and model_fields_set contains the field
        if self.sensitivity_depth_unit is None and "sensitivity_depth_unit" in self.model_fields_set:
            _dict['sensitivity_depth_unit'] = None

        # set to None if sensitivity_depth (nullable) is None
        # and model_fields_set contains the field
        if self.sensitivity_depth is None and "sensitivity_depth" in self.model_fields_set:
            _dict['sensitivity_depth'] = None

        # set to None if sensitivity_time_seconds (nullable) is None
        # and model_fields_set contains the field
        if self.sensitivity_time_seconds is None and "sensitivity_time_seconds" in self.model_fields_set:
            _dict['sensitivity_time_seconds'] = None

        # set to None if reference_url (nullable) is None
        # and model_fields_set contains the field
        if self.reference_url is None and "reference_url" in self.model_fields_set:
            _dict['reference_url'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Filter from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "created_on": obj.get("created_on"),
            "name": obj.get("name"),
            "peak_wavelength": obj.get("peak_wavelength"),
            "min_wavelength": obj.get("min_wavelength"),
            "max_wavelength": obj.get("max_wavelength"),
            "is_operational": obj.get("is_operational"),
            "sensitivity_depth_unit": obj.get("sensitivity_depth_unit"),
            "sensitivity_depth": obj.get("sensitivity_depth"),
            "sensitivity_time_seconds": obj.get("sensitivity_time_seconds"),
            "reference_url": obj.get("reference_url"),
            "instrument_id": obj.get("instrument_id")
        })
        return _obj





# coding: utf-8

"""
    ACROSS Server

    Server providing tools and utilities for various NASA missions to aid in coordination of large observation efforts.

    The version of the OpenAPI document: 1.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from datetime import datetime
from pydantic import Field, StrictBool, StrictStr
from typing import List, Optional
from typing_extensions import Annotated
from across.sdk.v1.models.page_schedule import PageSchedule
from across.sdk.v1.models.schedule import Schedule
from across.sdk.v1.models.schedule_create import ScheduleCreate
from across.sdk.v1.models.schedule_create_many import ScheduleCreateMany
from across.sdk.v1.models.schedule_fidelity import ScheduleFidelity
from across.sdk.v1.models.schedule_status import ScheduleStatus

from across.sdk.v1.api_client import ApiClient, RequestSerialized
from across.sdk.v1.api_response import ApiResponse
from across.sdk.v1.rest import RESTResponseType


class ScheduleApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def create_many_schedules(
        self,
        schedule_create_many: ScheduleCreateMany,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[str]:
        """Create many Schedules

        Create many new observing schedules for ACROSS.

        :param schedule_create_many: (required)
        :type schedule_create_many: ScheduleCreateMany
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_many_schedules_serialize(
            schedule_create_many=schedule_create_many,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "List[str]",
            '404': None,
            '422': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def create_many_schedules_with_http_info(
        self,
        schedule_create_many: ScheduleCreateMany,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[str]]:
        """Create many Schedules

        Create many new observing schedules for ACROSS.

        :param schedule_create_many: (required)
        :type schedule_create_many: ScheduleCreateMany
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_many_schedules_serialize(
            schedule_create_many=schedule_create_many,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "List[str]",
            '404': None,
            '422': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def create_many_schedules_without_preload_content(
        self,
        schedule_create_many: ScheduleCreateMany,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Create many Schedules

        Create many new observing schedules for ACROSS.

        :param schedule_create_many: (required)
        :type schedule_create_many: ScheduleCreateMany
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_many_schedules_serialize(
            schedule_create_many=schedule_create_many,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "List[str]",
            '404': None,
            '422': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _create_many_schedules_serialize(
        self,
        schedule_create_many,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if schedule_create_many is not None:
            _body_params = schedule_create_many


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Authorization'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/schedule/bulk',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def create_schedule(
        self,
        schedule_create: ScheduleCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> str:
        """Create a Schedule

        Create a new observing schedule for ACROSS.

        :param schedule_create: (required)
        :type schedule_create: ScheduleCreate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_schedule_serialize(
            schedule_create=schedule_create,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "str",
            '404': None,
            '409': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def create_schedule_with_http_info(
        self,
        schedule_create: ScheduleCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[str]:
        """Create a Schedule

        Create a new observing schedule for ACROSS.

        :param schedule_create: (required)
        :type schedule_create: ScheduleCreate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_schedule_serialize(
            schedule_create=schedule_create,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "str",
            '404': None,
            '409': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def create_schedule_without_preload_content(
        self,
        schedule_create: ScheduleCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Create a Schedule

        Create a new observing schedule for ACROSS.

        :param schedule_create: (required)
        :type schedule_create: ScheduleCreate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_schedule_serialize(
            schedule_create=schedule_create,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "str",
            '404': None,
            '409': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _create_schedule_serialize(
        self,
        schedule_create,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if schedule_create is not None:
            _body_params = schedule_create


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Authorization'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/schedule/',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_schedule(
        self,
        schedule_id: StrictStr,
        include_observations: Optional[StrictBool] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Schedule:
        """Read a schedule

        Read a schedule by a schedule ID.

        :param schedule_id: (required)
        :type schedule_id: str
        :param include_observations:
        :type include_observations: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_schedule_serialize(
            schedule_id=schedule_id,
            include_observations=include_observations,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Schedule",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_schedule_with_http_info(
        self,
        schedule_id: StrictStr,
        include_observations: Optional[StrictBool] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Schedule]:
        """Read a schedule

        Read a schedule by a schedule ID.

        :param schedule_id: (required)
        :type schedule_id: str
        :param include_observations:
        :type include_observations: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_schedule_serialize(
            schedule_id=schedule_id,
            include_observations=include_observations,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Schedule",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_schedule_without_preload_content(
        self,
        schedule_id: StrictStr,
        include_observations: Optional[StrictBool] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Read a schedule

        Read a schedule by a schedule ID.

        :param schedule_id: (required)
        :type schedule_id: str
        :param include_observations:
        :type include_observations: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_schedule_serialize(
            schedule_id=schedule_id,
            include_observations=include_observations,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Schedule",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_schedule_serialize(
        self,
        schedule_id,
        include_observations,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if schedule_id is not None:
            _path_params['schedule_id'] = schedule_id
        # process the query parameters
        if include_observations is not None:
            
            _query_params.append(('include_observations', include_observations))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/schedule/{schedule_id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_schedules(
        self,
        page: Annotated[Optional[Annotated[int, Field(strict=True, ge=1)]], Field(description="Page number")] = None,
        page_limit: Annotated[Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]], Field(description="Records per page")] = None,
        date_range_begin: Optional[datetime] = None,
        date_range_end: Optional[datetime] = None,
        status: Optional[ScheduleStatus] = None,
        external_id: Optional[StrictStr] = None,
        fidelity: Optional[ScheduleFidelity] = None,
        created_on: Optional[datetime] = None,
        observatory_ids: Optional[List[Optional[StrictStr]]] = None,
        observatory_names: Optional[List[Optional[StrictStr]]] = None,
        telescope_ids: Optional[List[Optional[StrictStr]]] = None,
        telescope_names: Optional[List[Optional[StrictStr]]] = None,
        name: Optional[StrictStr] = None,
        include_observations: Optional[StrictBool] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PageSchedule:
        """Read schedule(s)

        Read most recent schedules based on query params

        :param page: Page number
        :type page: int
        :param page_limit: Records per page
        :type page_limit: int
        :param date_range_begin:
        :type date_range_begin: datetime
        :param date_range_end:
        :type date_range_end: datetime
        :param status:
        :type status: ScheduleStatus
        :param external_id:
        :type external_id: str
        :param fidelity:
        :type fidelity: ScheduleFidelity
        :param created_on:
        :type created_on: datetime
        :param observatory_ids:
        :type observatory_ids: List[Optional[str]]
        :param observatory_names:
        :type observatory_names: List[Optional[str]]
        :param telescope_ids:
        :type telescope_ids: List[Optional[str]]
        :param telescope_names:
        :type telescope_names: List[Optional[str]]
        :param name:
        :type name: str
        :param include_observations:
        :type include_observations: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_schedules_serialize(
            page=page,
            page_limit=page_limit,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            status=status,
            external_id=external_id,
            fidelity=fidelity,
            created_on=created_on,
            observatory_ids=observatory_ids,
            observatory_names=observatory_names,
            telescope_ids=telescope_ids,
            telescope_names=telescope_names,
            name=name,
            include_observations=include_observations,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PageSchedule",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_schedules_with_http_info(
        self,
        page: Annotated[Optional[Annotated[int, Field(strict=True, ge=1)]], Field(description="Page number")] = None,
        page_limit: Annotated[Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]], Field(description="Records per page")] = None,
        date_range_begin: Optional[datetime] = None,
        date_range_end: Optional[datetime] = None,
        status: Optional[ScheduleStatus] = None,
        external_id: Optional[StrictStr] = None,
        fidelity: Optional[ScheduleFidelity] = None,
        created_on: Optional[datetime] = None,
        observatory_ids: Optional[List[Optional[StrictStr]]] = None,
        observatory_names: Optional[List[Optional[StrictStr]]] = None,
        telescope_ids: Optional[List[Optional[StrictStr]]] = None,
        telescope_names: Optional[List[Optional[StrictStr]]] = None,
        name: Optional[StrictStr] = None,
        include_observations: Optional[StrictBool] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PageSchedule]:
        """Read schedule(s)

        Read most recent schedules based on query params

        :param page: Page number
        :type page: int
        :param page_limit: Records per page
        :type page_limit: int
        :param date_range_begin:
        :type date_range_begin: datetime
        :param date_range_end:
        :type date_range_end: datetime
        :param status:
        :type status: ScheduleStatus
        :param external_id:
        :type external_id: str
        :param fidelity:
        :type fidelity: ScheduleFidelity
        :param created_on:
        :type created_on: datetime
        :param observatory_ids:
        :type observatory_ids: List[Optional[str]]
        :param observatory_names:
        :type observatory_names: List[Optional[str]]
        :param telescope_ids:
        :type telescope_ids: List[Optional[str]]
        :param telescope_names:
        :type telescope_names: List[Optional[str]]
        :param name:
        :type name: str
        :param include_observations:
        :type include_observations: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_schedules_serialize(
            page=page,
            page_limit=page_limit,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            status=status,
            external_id=external_id,
            fidelity=fidelity,
            created_on=created_on,
            observatory_ids=observatory_ids,
            observatory_names=observatory_names,
            telescope_ids=telescope_ids,
            telescope_names=telescope_names,
            name=name,
            include_observations=include_observations,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PageSchedule",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_schedules_without_preload_content(
        self,
        page: Annotated[Optional[Annotated[int, Field(strict=True, ge=1)]], Field(description="Page number")] = None,
        page_limit: Annotated[Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]], Field(description="Records per page")] = None,
        date_range_begin: Optional[datetime] = None,
        date_range_end: Optional[datetime] = None,
        status: Optional[ScheduleStatus] = None,
        external_id: Optional[StrictStr] = None,
        fidelity: Optional[ScheduleFidelity] = None,
        created_on: Optional[datetime] = None,
        observatory_ids: Optional[List[Optional[StrictStr]]] = None,
        observatory_names: Optional[List[Optional[StrictStr]]] = None,
        telescope_ids: Optional[List[Optional[StrictStr]]] = None,
        telescope_names: Optional[List[Optional[StrictStr]]] = None,
        name: Optional[StrictStr] = None,
        include_observations: Optional[StrictBool] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Read schedule(s)

        Read most recent schedules based on query params

        :param page: Page number
        :type page: int
        :param page_limit: Records per page
        :type page_limit: int
        :param date_range_begin:
        :type date_range_begin: datetime
        :param date_range_end:
        :type date_range_end: datetime
        :param status:
        :type status: ScheduleStatus
        :param external_id:
        :type external_id: str
        :param fidelity:
        :type fidelity: ScheduleFidelity
        :param created_on:
        :type created_on: datetime
        :param observatory_ids:
        :type observatory_ids: List[Optional[str]]
        :param observatory_names:
        :type observatory_names: List[Optional[str]]
        :param telescope_ids:
        :type telescope_ids: List[Optional[str]]
        :param telescope_names:
        :type telescope_names: List[Optional[str]]
        :param name:
        :type name: str
        :param include_observations:
        :type include_observations: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_schedules_serialize(
            page=page,
            page_limit=page_limit,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            status=status,
            external_id=external_id,
            fidelity=fidelity,
            created_on=created_on,
            observatory_ids=observatory_ids,
            observatory_names=observatory_names,
            telescope_ids=telescope_ids,
            telescope_names=telescope_names,
            name=name,
            include_observations=include_observations,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PageSchedule",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_schedules_serialize(
        self,
        page,
        page_limit,
        date_range_begin,
        date_range_end,
        status,
        external_id,
        fidelity,
        created_on,
        observatory_ids,
        observatory_names,
        telescope_ids,
        telescope_names,
        name,
        include_observations,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'observatory_ids': 'multi',
            'observatory_names': 'multi',
            'telescope_ids': 'multi',
            'telescope_names': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if page is not None:
            
            _query_params.append(('page', page))
            
        if page_limit is not None:
            
            _query_params.append(('page_limit', page_limit))
            
        if date_range_begin is not None:
            if isinstance(date_range_begin, datetime):
                _query_params.append(
                    (
                        'date_range_begin',
                        date_range_begin.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('date_range_begin', date_range_begin))
            
        if date_range_end is not None:
            if isinstance(date_range_end, datetime):
                _query_params.append(
                    (
                        'date_range_end',
                        date_range_end.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('date_range_end', date_range_end))
            
        if status is not None:
            
            _query_params.append(('status', status.value))
            
        if external_id is not None:
            
            _query_params.append(('external_id', external_id))
            
        if fidelity is not None:
            
            _query_params.append(('fidelity', fidelity.value))
            
        if created_on is not None:
            if isinstance(created_on, datetime):
                _query_params.append(
                    (
                        'created_on',
                        created_on.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('created_on', created_on))
            
        if observatory_ids is not None:
            
            _query_params.append(('observatory_ids', observatory_ids))
            
        if observatory_names is not None:
            
            _query_params.append(('observatory_names', observatory_names))
            
        if telescope_ids is not None:
            
            _query_params.append(('telescope_ids', telescope_ids))
            
        if telescope_names is not None:
            
            _query_params.append(('telescope_names', telescope_names))
            
        if name is not None:
            
            _query_params.append(('name', name))
            
        if include_observations is not None:
            
            _query_params.append(('include_observations', include_observations))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/schedule/',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_schedules_history(
        self,
        page: Annotated[Optional[Annotated[int, Field(strict=True, ge=1)]], Field(description="Page number")] = None,
        page_limit: Annotated[Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]], Field(description="Records per page")] = None,
        date_range_begin: Optional[datetime] = None,
        date_range_end: Optional[datetime] = None,
        status: Optional[ScheduleStatus] = None,
        external_id: Optional[StrictStr] = None,
        fidelity: Optional[ScheduleFidelity] = None,
        created_on: Optional[datetime] = None,
        observatory_ids: Optional[List[Optional[StrictStr]]] = None,
        observatory_names: Optional[List[Optional[StrictStr]]] = None,
        telescope_ids: Optional[List[Optional[StrictStr]]] = None,
        telescope_names: Optional[List[Optional[StrictStr]]] = None,
        name: Optional[StrictStr] = None,
        include_observations: Optional[StrictBool] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PageSchedule:
        """Read schedule(s)

        Read many recent schedules based on query params

        :param page: Page number
        :type page: int
        :param page_limit: Records per page
        :type page_limit: int
        :param date_range_begin:
        :type date_range_begin: datetime
        :param date_range_end:
        :type date_range_end: datetime
        :param status:
        :type status: ScheduleStatus
        :param external_id:
        :type external_id: str
        :param fidelity:
        :type fidelity: ScheduleFidelity
        :param created_on:
        :type created_on: datetime
        :param observatory_ids:
        :type observatory_ids: List[Optional[str]]
        :param observatory_names:
        :type observatory_names: List[Optional[str]]
        :param telescope_ids:
        :type telescope_ids: List[Optional[str]]
        :param telescope_names:
        :type telescope_names: List[Optional[str]]
        :param name:
        :type name: str
        :param include_observations:
        :type include_observations: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_schedules_history_serialize(
            page=page,
            page_limit=page_limit,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            status=status,
            external_id=external_id,
            fidelity=fidelity,
            created_on=created_on,
            observatory_ids=observatory_ids,
            observatory_names=observatory_names,
            telescope_ids=telescope_ids,
            telescope_names=telescope_names,
            name=name,
            include_observations=include_observations,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PageSchedule",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_schedules_history_with_http_info(
        self,
        page: Annotated[Optional[Annotated[int, Field(strict=True, ge=1)]], Field(description="Page number")] = None,
        page_limit: Annotated[Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]], Field(description="Records per page")] = None,
        date_range_begin: Optional[datetime] = None,
        date_range_end: Optional[datetime] = None,
        status: Optional[ScheduleStatus] = None,
        external_id: Optional[StrictStr] = None,
        fidelity: Optional[ScheduleFidelity] = None,
        created_on: Optional[datetime] = None,
        observatory_ids: Optional[List[Optional[StrictStr]]] = None,
        observatory_names: Optional[List[Optional[StrictStr]]] = None,
        telescope_ids: Optional[List[Optional[StrictStr]]] = None,
        telescope_names: Optional[List[Optional[StrictStr]]] = None,
        name: Optional[StrictStr] = None,
        include_observations: Optional[StrictBool] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PageSchedule]:
        """Read schedule(s)

        Read many recent schedules based on query params

        :param page: Page number
        :type page: int
        :param page_limit: Records per page
        :type page_limit: int
        :param date_range_begin:
        :type date_range_begin: datetime
        :param date_range_end:
        :type date_range_end: datetime
        :param status:
        :type status: ScheduleStatus
        :param external_id:
        :type external_id: str
        :param fidelity:
        :type fidelity: ScheduleFidelity
        :param created_on:
        :type created_on: datetime
        :param observatory_ids:
        :type observatory_ids: List[Optional[str]]
        :param observatory_names:
        :type observatory_names: List[Optional[str]]
        :param telescope_ids:
        :type telescope_ids: List[Optional[str]]
        :param telescope_names:
        :type telescope_names: List[Optional[str]]
        :param name:
        :type name: str
        :param include_observations:
        :type include_observations: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_schedules_history_serialize(
            page=page,
            page_limit=page_limit,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            status=status,
            external_id=external_id,
            fidelity=fidelity,
            created_on=created_on,
            observatory_ids=observatory_ids,
            observatory_names=observatory_names,
            telescope_ids=telescope_ids,
            telescope_names=telescope_names,
            name=name,
            include_observations=include_observations,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PageSchedule",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_schedules_history_without_preload_content(
        self,
        page: Annotated[Optional[Annotated[int, Field(strict=True, ge=1)]], Field(description="Page number")] = None,
        page_limit: Annotated[Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]], Field(description="Records per page")] = None,
        date_range_begin: Optional[datetime] = None,
        date_range_end: Optional[datetime] = None,
        status: Optional[ScheduleStatus] = None,
        external_id: Optional[StrictStr] = None,
        fidelity: Optional[ScheduleFidelity] = None,
        created_on: Optional[datetime] = None,
        observatory_ids: Optional[List[Optional[StrictStr]]] = None,
        observatory_names: Optional[List[Optional[StrictStr]]] = None,
        telescope_ids: Optional[List[Optional[StrictStr]]] = None,
        telescope_names: Optional[List[Optional[StrictStr]]] = None,
        name: Optional[StrictStr] = None,
        include_observations: Optional[StrictBool] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Read schedule(s)

        Read many recent schedules based on query params

        :param page: Page number
        :type page: int
        :param page_limit: Records per page
        :type page_limit: int
        :param date_range_begin:
        :type date_range_begin: datetime
        :param date_range_end:
        :type date_range_end: datetime
        :param status:
        :type status: ScheduleStatus
        :param external_id:
        :type external_id: str
        :param fidelity:
        :type fidelity: ScheduleFidelity
        :param created_on:
        :type created_on: datetime
        :param observatory_ids:
        :type observatory_ids: List[Optional[str]]
        :param observatory_names:
        :type observatory_names: List[Optional[str]]
        :param telescope_ids:
        :type telescope_ids: List[Optional[str]]
        :param telescope_names:
        :type telescope_names: List[Optional[str]]
        :param name:
        :type name: str
        :param include_observations:
        :type include_observations: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_schedules_history_serialize(
            page=page,
            page_limit=page_limit,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            status=status,
            external_id=external_id,
            fidelity=fidelity,
            created_on=created_on,
            observatory_ids=observatory_ids,
            observatory_names=observatory_names,
            telescope_ids=telescope_ids,
            telescope_names=telescope_names,
            name=name,
            include_observations=include_observations,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PageSchedule",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_schedules_history_serialize(
        self,
        page,
        page_limit,
        date_range_begin,
        date_range_end,
        status,
        external_id,
        fidelity,
        created_on,
        observatory_ids,
        observatory_names,
        telescope_ids,
        telescope_names,
        name,
        include_observations,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'observatory_ids': 'multi',
            'observatory_names': 'multi',
            'telescope_ids': 'multi',
            'telescope_names': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if page is not None:
            
            _query_params.append(('page', page))
            
        if page_limit is not None:
            
            _query_params.append(('page_limit', page_limit))
            
        if date_range_begin is not None:
            if isinstance(date_range_begin, datetime):
                _query_params.append(
                    (
                        'date_range_begin',
                        date_range_begin.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('date_range_begin', date_range_begin))
            
        if date_range_end is not None:
            if isinstance(date_range_end, datetime):
                _query_params.append(
                    (
                        'date_range_end',
                        date_range_end.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('date_range_end', date_range_end))
            
        if status is not None:
            
            _query_params.append(('status', status.value))
            
        if external_id is not None:
            
            _query_params.append(('external_id', external_id))
            
        if fidelity is not None:
            
            _query_params.append(('fidelity', fidelity.value))
            
        if created_on is not None:
            if isinstance(created_on, datetime):
                _query_params.append(
                    (
                        'created_on',
                        created_on.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('created_on', created_on))
            
        if observatory_ids is not None:
            
            _query_params.append(('observatory_ids', observatory_ids))
            
        if observatory_names is not None:
            
            _query_params.append(('observatory_names', observatory_names))
            
        if telescope_ids is not None:
            
            _query_params.append(('telescope_ids', telescope_ids))
            
        if telescope_names is not None:
            
            _query_params.append(('telescope_names', telescope_names))
            
        if name is not None:
            
            _query_params.append(('name', name))
            
        if include_observations is not None:
            
            _query_params.append(('include_observations', include_observations))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/schedule/history',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

"""
    ACROSS Server

    Server providing tools and utilities for various NASA missions to aid in coordination of large observation efforts.

    The version of the OpenAPI document: 1.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List
from across.sdk.v1.models.visibility_window import VisibilityWindow
from typing import Optional, Set
from typing_extensions import Self

class JointVisibilityResult(BaseModel):
    """
    A Pydantic model class representing the joint visibility calculation parameters.  This class is used for joint visibility calculations in the ACROSS SSA system.  Parameters ---------- instrument_ids: list[UUID]     List of instrument IDs to check visibility against visibility_windows: list[VisibilityWindow]     List of joint visibility windows for all the instruments observatory_visibility_windows: dict[UUID, list[VisibilityWindow]]     Dictionary mapping instrument IDs to their respective visibility windows
    """ # noqa: E501
    instrument_ids: List[StrictStr]
    visibility_windows: List[VisibilityWindow]
    observatory_visibility_windows: Dict[str, List[VisibilityWindow]]
    __properties: ClassVar[List[str]] = ["instrument_ids", "visibility_windows", "observatory_visibility_windows"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of JointVisibilityResult from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in visibility_windows (list)
        _items = []
        if self.visibility_windows:
            for _item_visibility_windows in self.visibility_windows:
                if _item_visibility_windows:
                    _items.append(_item_visibility_windows.to_dict())
            _dict['visibility_windows'] = _items
        # override the default output from pydantic by calling `to_dict()` of each value in observatory_visibility_windows (dict of array)
        _field_dict_of_array = {}
        if self.observatory_visibility_windows:
            for _key_observatory_visibility_windows in self.observatory_visibility_windows:
                if self.observatory_visibility_windows[_key_observatory_visibility_windows] is not None:
                    _field_dict_of_array[_key_observatory_visibility_windows] = [
                        _item.to_dict() for _item in self.observatory_visibility_windows[_key_observatory_visibility_windows]
                    ]
            _dict['observatory_visibility_windows'] = _field_dict_of_array
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of JointVisibilityResult from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "instrument_ids": obj.get("instrument_ids"),
            "visibility_windows": [VisibilityWindow.from_dict(_item) for _item in obj["visibility_windows"]] if obj.get("visibility_windows") is not None else None,
            "observatory_visibility_windows": dict(
                (_k,
                        [VisibilityWindow.from_dict(_item) for _item in _v]
                        if _v is not None
                        else None
                )
                for _k, _v in obj.get("observatory_visibility_windows", {}).items()
            )
        })
        return _obj





# coding: utf-8

"""
    ACROSS Server

    Server providing tools and utilities for various NASA missions to aid in coordination of large observation efforts.

    The version of the OpenAPI document: 1.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from across.sdk.v1.models.filter import Filter
from across.sdk.v1.models.id_name_schema import IDNameSchema
from across.sdk.v1.models.instrument_constraints_inner import InstrumentConstraintsInner
from across.sdk.v1.models.point import Point
from across.sdk.v1.models.visibility_type import VisibilityType
from typing import Optional, Set
from typing_extensions import Self

class Instrument(BaseModel):
    """
    A Pydantic model class representing a created Instrument  Notes ----- Inherits from InstrumentBase  Methods ------- from_orm(instrument: InstrumentModel) -> Instrument     Static method that instantiates this class from a Instrument database record
    """ # noqa: E501
    id: StrictStr
    created_on: datetime
    name: StrictStr
    short_name: StrictStr
    telescope: Optional[IDNameSchema] = None
    footprints: Optional[List[List[Point]]] = None
    filters: Optional[List[Filter]] = None
    constraints: Optional[List[InstrumentConstraintsInner]] = None
    visibility_type: Optional[VisibilityType] = None
    __properties: ClassVar[List[str]] = ["id", "created_on", "name", "short_name", "telescope", "footprints", "filters", "constraints", "visibility_type"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Instrument from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of telescope
        if self.telescope:
            _dict['telescope'] = self.telescope.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in footprints (list of list)
        _items = []
        if self.footprints:
            for _item_footprints in self.footprints:
                if _item_footprints:
                    _items.append(
                         [_inner_item.to_dict() for _inner_item in _item_footprints if _inner_item is not None]
                    )
            _dict['footprints'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in filters (list)
        _items = []
        if self.filters:
            for _item_filters in self.filters:
                if _item_filters:
                    _items.append(_item_filters.to_dict())
            _dict['filters'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in constraints (list)
        _items = []
        if self.constraints:
            for _item_constraints in self.constraints:
                if _item_constraints:
                    _items.append(_item_constraints.to_dict())
            _dict['constraints'] = _items
        # set to None if telescope (nullable) is None
        # and model_fields_set contains the field
        if self.telescope is None and "telescope" in self.model_fields_set:
            _dict['telescope'] = None

        # set to None if footprints (nullable) is None
        # and model_fields_set contains the field
        if self.footprints is None and "footprints" in self.model_fields_set:
            _dict['footprints'] = None

        # set to None if filters (nullable) is None
        # and model_fields_set contains the field
        if self.filters is None and "filters" in self.model_fields_set:
            _dict['filters'] = None

        # set to None if constraints (nullable) is None
        # and model_fields_set contains the field
        if self.constraints is None and "constraints" in self.model_fields_set:
            _dict['constraints'] = None

        # set to None if visibility_type (nullable) is None
        # and model_fields_set contains the field
        if self.visibility_type is None and "visibility_type" in self.model_fields_set:
            _dict['visibility_type'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Instrument from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "created_on": obj.get("created_on"),
            "name": obj.get("name"),
            "short_name": obj.get("short_name"),
            "telescope": IDNameSchema.from_dict(obj["telescope"]) if obj.get("telescope") is not None else None,
            "footprints": [
                    [Point.from_dict(_inner_item) for _inner_item in _item]
                    for _item in obj["footprints"]
                ] if obj.get("footprints") is not None else None,
            "filters": [Filter.from_dict(_item) for _item in obj["filters"]] if obj.get("filters") is not None else None,
            "constraints": [InstrumentConstraintsInner.from_dict(_item) for _item in obj["constraints"]] if obj.get("constraints") is not None else None,
            "visibility_type": obj.get("visibility_type")
        })
        return _obj





# coding: utf-8

"""
    ACROSS Server

    Server providing tools and utilities for various NASA missions to aid in coordination of large observation efforts.

    The version of the OpenAPI document: 1.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

class TLE(BaseModel):
    """
    TLE
    """ # noqa: E501
    norad_id: StrictInt
    satellite_name: StrictStr
    tle1: Annotated[str, Field(min_length=69, strict=True, max_length=69)]
    tle2: Annotated[str, Field(min_length=69, strict=True, max_length=69)]
    epoch: datetime = Field(description="Calculate the Epoch of the TLE file. See https://celestrak.org/columns/v04n03/#FAQ04 for more information on how the year / epoch encoding works.  Returns -------     The calculated epoch of the TLE.")
    __properties: ClassVar[List[str]] = ["norad_id", "satellite_name", "tle1", "tle2", "epoch"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TLE from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "epoch",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TLE from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "norad_id": obj.get("norad_id"),
            "satellite_name": obj.get("satellite_name"),
            "tle1": obj.get("tle1"),
            "tle2": obj.get("tle2"),
            "epoch": obj.get("epoch")
        })
        return _obj





# coding: utf-8

"""
    ACROSS Server

    Server providing tools and utilities for various NASA missions to aid in coordination of large observation efforts.

    The version of the OpenAPI document: 1.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class FrequencyUnit(str, Enum):
    """
    Enum to represent the bandpass frequency types
    """

    """
    allowed enum values
    """
    HZ = 'Hz'
    KHZ = 'kHz'
    MHZ = 'MHz'
    GHZ = 'GHz'
    THZ = 'THz'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of FrequencyUnit from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    ACROSS Server

    Server providing tools and utilities for various NASA missions to aid in coordination of large observation efforts.

    The version of the OpenAPI document: 1.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from across.sdk.v1.models.date_range import DateRange
from across.sdk.v1.models.observation_create import ObservationCreate
from across.sdk.v1.models.schedule_fidelity import ScheduleFidelity
from across.sdk.v1.models.schedule_status import ScheduleStatus
from typing import Optional, Set
from typing_extensions import Self

class ScheduleCreate(BaseModel):
    """
    A Pydantic model class representing the a schedule to be created in the database  Parameters ---------- observations : list[schemas.Observation]     A list of observations for the schedule  Notes --------- Inherits from ScheduleBase  Methods to_orm(self, created_by_id: UUID) -> ScheduleModel     Method that creates the ORM record for a schedule to be serialized into the database.     This method does not instantiate the list of observations, the observation schema requires     a schedule ID so it is instantiated after the model id is flushed within the service.
    """ # noqa: E501
    telescope_id: StrictStr
    name: StrictStr
    date_range: DateRange
    status: ScheduleStatus
    external_id: Optional[StrictStr] = None
    fidelity: Optional[ScheduleFidelity] = None
    observations: List[ObservationCreate]
    __properties: ClassVar[List[str]] = ["telescope_id", "name", "date_range", "status", "external_id", "fidelity", "observations"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ScheduleCreate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of date_range
        if self.date_range:
            _dict['date_range'] = self.date_range.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in observations (list)
        _items = []
        if self.observations:
            for _item_observations in self.observations:
                if _item_observations:
                    _items.append(_item_observations.to_dict())
            _dict['observations'] = _items
        # set to None if external_id (nullable) is None
        # and model_fields_set contains the field
        if self.external_id is None and "external_id" in self.model_fields_set:
            _dict['external_id'] = None

        # set to None if fidelity (nullable) is None
        # and model_fields_set contains the field
        if self.fidelity is None and "fidelity" in self.model_fields_set:
            _dict['fidelity'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ScheduleCreate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "telescope_id": obj.get("telescope_id"),
            "name": obj.get("name"),
            "date_range": DateRange.from_dict(obj["date_range"]) if obj.get("date_range") is not None else None,
            "status": obj.get("status"),
            "external_id": obj.get("external_id"),
            "fidelity": obj.get("fidelity"),
            "observations": [ObservationCreate.from_dict(_item) for _item in obj["observations"]] if obj.get("observations") is not None else None
        })
        return _obj





# flake8: noqa

# import apis into api package
from across.sdk.v1.api.auth_api import AuthApi
from across.sdk.v1.api.filter_api import FilterApi
from across.sdk.v1.api.group_api import GroupApi
from across.sdk.v1.api.group_invite_api import GroupInviteApi
from across.sdk.v1.api.group_role_api import GroupRoleApi
from across.sdk.v1.api.instrument_api import InstrumentApi
from across.sdk.v1.api.internal_api import InternalApi
from across.sdk.v1.api.observation_api import ObservationApi
from across.sdk.v1.api.observatory_api import ObservatoryApi
from across.sdk.v1.api.permission_api import PermissionApi
from across.sdk.v1.api.schedule_api import ScheduleApi
from across.sdk.v1.api.service_account_api import ServiceAccountApi
from across.sdk.v1.api.tle_api import TLEApi
from across.sdk.v1.api.telescope_api import TelescopeApi
from across.sdk.v1.api.tools_api import ToolsApi
from across.sdk.v1.api.user_api import UserApi




import threading
import logging
import os
import base64
import json
from datetime import datetime, timedelta, timezone
import time
from typing import Any

import across.sdk.v1 as sdk
import across.sdk.v1.exceptions as sdk_exceptions
from across.sdk.v1 import rest

from .abstract_credential_storage import CredentialStorage as ICredStorage

logger = logging.getLogger("ACROSS_API_CLIENT_WRAPPER")


class ApiClientWrapper(sdk.ApiClient):
    _client = None
    _cred_store: ICredStorage | None = None
    _exp: datetime | None = None

    def __init__(
        self,
        configuration: sdk.Configuration,
        creds_store: ICredStorage | None = None,
        *args,
        **kwargs,
    ):
        super().__init__(configuration, *args, **kwargs)

        self._cred_store = creds_store
        self._lock = threading.Lock()

    @classmethod
    def get_client(
        cls,
        configuration: sdk.Configuration,
        creds: ICredStorage | None = None,
    ):
        """
        Retrieve (or lazily initialize) a singleton API client.

        This method ensures that only one client instance is created per class.
        If no client exists, it will initialize one using the provided
        `configuration` and optional `creds` (credentials storage).

        Credentials resolution order:
            1. Use `configuration.username` and `configuration.password` if they are provided.
            2. If not provided and `creds` is passed in, fetch credentials from it.
            3. Otherwise, fall back to environment variables:
                - `ACROSS_SERVER_ID`
                - `ACROSS_SERVER_SECRET`

        ⚠️ Note:
            This is a **singleton** accessor. Once the client is created,
            subsequent calls to `get_client` will reuse the same instance,
            regardless of whether a different `configuration` or `creds` is passed.
            To reset or replace the client, `_client` must be cleared explicitly.

        Args:
            configuration (sdk.Configuration): The base configuration object
                for the SDK client.
            creds (ICredStorage | None, optional): Optional credential storage
                provider for resolving `id` and `secret`. Defaults to None.

        Returns:
            ApiClientWrapper: A singleton instance of the API client wrapper.

        Example:
                from my_sdk import Configuration, ApiClientWrapper, CredentialStorage, SomeApi
                from my_creds import CredStorage

                config = Configuration(host="https://api.example.com")
                creds: CredentialStorage = CredStorage()

                client = ApiClientWrapper.get_client(configuration=config, creds=creds)

                response = SomeApi(client).some_method()
                print(response)
        """
        if cls._client is None:
            if not configuration.username and not configuration.password:
                # Use the creds store if it is passed in, otherwise use env vars
                if creds:
                    logger.debug(
                        "Force retrieve credentials from provided credential store."
                    )
                    configuration.username = creds.id(force=True)
                    configuration.password = creds.secret(force=True)
                else:
                    configuration.username = os.getenv("ACROSS_SERVER_ID")
                    configuration.password = os.getenv("ACROSS_SERVER_SECRET")

            cls._client = ApiClientWrapper(
                configuration=configuration,
                creds_store=creds,
            )

        return cls._client

    def param_serialize(self, *args, **kwargs) -> Any:
        """
        Override param_serialize to refresh token before non-GET calls.

        :return: Serialized parameters, will be RequestSerialized from ApiClient but it is not exported.
        """
        # API class routers (e.g. sdk.ScheduleApi), will serialize params before calling `call_api`.
        # Ensure token is fresh before making non-GET calls.
        if kwargs["method"].lower() != "get":
            self.refresh()

        return super().param_serialize(*args, **kwargs)

    def call_api(self, *args, **kwargs) -> rest.RESTResponse:
        try:
            return super().call_api(*args, **kwargs)
        except (sdk.ApiException, sdk_exceptions.UnauthorizedException) as err:
            if err.status == 401:
                logger.debug("Access token is unauthenticated or it has expired.")

                refreshed = self.refresh_token()

                # Attempt the call again
                if refreshed:
                    # Reset the Authorization header with the new token and retry
                    args[2]["Authorization"] = (
                        f"Bearer {self.configuration.access_token}"
                    )
                    logger.debug("Retrying API call with refreshed token...")
                    return super().call_api(*args, **kwargs)
                else:
                    raise err
            else:
                raise err

    def refresh(self) -> None:
        if not self.configuration.access_token:
            logger.debug("No access_token.")
            self.refresh_token()

        if self._is_token_invalid(self.configuration.access_token):
            logger.debug("Expired access_token.")
            self.refresh_token()

        if self._cred_store:
            with self._lock:
                if self._should_rotate(self._cred_store):
                    res = sdk.InternalApi(super()).service_account_rotate_key(
                        service_account_id=self._cred_store.id()
                    )

                    self._set_exp(res.expiration)
                    self._cred_store.update_key(res.secret_key)
                    self.configuration.password = res.secret_key

    def _decode_jwt_part(self, encoded_part) -> dict[str, Any]:
        """Decodes a Base64Url-encoded JWT part and returns the decoded JSON as a dictionary."""
        # Add padding characters if missing, as Base64Url encoding might omit them.
        # Base64 requires padding to be a multiple of 4.
        missing_padding = len(encoded_part) % 4
        if missing_padding != 0:
            encoded_part += "=" * (4 - missing_padding)

        # Decode from Base64Url to bytes to UTF-8
        decoded_bytes = base64.urlsafe_b64decode(encoded_part).decode("utf-8")

        if decoded_bytes is None:
            logger.debug("Could not decode jwt payload as bytes")
            return {}

        decoded_json = json.loads(decoded_bytes)
        return decoded_json

    def _is_token_invalid(self, jwt_token):
        """Returns True when the token is expired, malformed, or missing expiration"""
        # JWT contains 3 parts, we're looking for the middle part; the payload with the exp key
        if not isinstance(jwt_token, str):
            return True

        jwt_parts = jwt_token.split(".")

        if len(jwt_parts) != 3:
            return True

        payload_encoded = jwt_parts[1]

        if payload_encoded is None:
            logger.debug("Token missing payload")
            return True

        payload = self._decode_jwt_part(payload_encoded)
        token_exp = payload.get("exp")
        current_timestamp = time.time()

        if token_exp is None:
            logger.debug("Token missing exp")
            return True

        # Add 30 seconds to avoid boundary condition expiry while request is in flight
        if token_exp < current_timestamp + 30:
            logger.debug("Token is expired")
            return True

        return False

    def refresh_token(self) -> bool:
        if self.configuration.username and self.configuration.password:
            logger.debug("Refreshing access token...")

            # Instantiate with super to avoid infinite recursion through call_api
            token = sdk.AuthApi(super()).token(
                grant_type=sdk.GrantType.CLIENT_CREDENTIALS
            )

            self.configuration.access_token = token.access_token

            logger.debug("Successfully refreshed token!")

            return True

        return False

    def _should_rotate(self, cred_store: ICredStorage) -> bool:
        now = datetime.now(timezone.utc)

        expiration = self._expiration(cred_store)

        if expiration:
            will_expire_soon = expiration <= now + timedelta(
                days=cred_store.days_before_exp
            )

            return will_expire_soon

        return True

    def _expiration(self, cred_store: ICredStorage) -> datetime | None:
        if self._exp:
            return self._exp

        res = sdk.InternalApi(super()).get_service_account(
            service_account_id=cred_store.id()
        )

        self._set_exp(res.expiration)

        return self._exp

    def _set_exp(self, date: datetime):
        if date.tzinfo is None:
            self._exp = date.replace(tzinfo=timezone.utc)
        else:
            self._exp = date



# coding: utf-8

"""
    ACROSS Server

    Server providing tools and utilities for various NASA missions to aid in coordination of large observation efforts.

    The version of the OpenAPI document: 1.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from across.sdk.v1.models.bandpass import Bandpass
from across.sdk.v1.models.coordinate import Coordinate
from across.sdk.v1.models.date_range import DateRange
from across.sdk.v1.models.ivoa_obs_category import IVOAObsCategory
from across.sdk.v1.models.ivoa_obs_tracking_type import IVOAObsTrackingType
from across.sdk.v1.models.observation_status import ObservationStatus
from across.sdk.v1.models.observation_type import ObservationType
from across.sdk.v1.models.unit_value import UnitValue
from typing import Optional, Set
from typing_extensions import Self

class ObservationCreate(BaseModel):
    """
    ObservationCreate
    """ # noqa: E501
    instrument_id: StrictStr
    object_name: StrictStr
    pointing_position: Optional[Coordinate] = None
    date_range: DateRange
    external_observation_id: StrictStr
    type: ObservationType
    status: ObservationStatus
    pointing_angle: Optional[Union[StrictFloat, StrictInt]] = None
    exposure_time: Optional[Union[StrictFloat, StrictInt]] = None
    reason: Optional[StrictStr] = None
    description: Optional[StrictStr] = None
    proposal_reference: Optional[StrictStr] = None
    object_position: Optional[Coordinate] = None
    depth: Optional[UnitValue] = None
    bandpass: Bandpass
    t_resolution: Optional[Union[StrictFloat, StrictInt]] = None
    em_res_power: Optional[Union[StrictFloat, StrictInt]] = None
    o_ucd: Optional[StrictStr] = None
    pol_states: Optional[StrictStr] = None
    pol_xel: Optional[StrictStr] = None
    category: Optional[IVOAObsCategory] = None
    priority: Optional[StrictInt] = None
    tracking_type: Optional[IVOAObsTrackingType] = None
    created_on: Optional[datetime] = None
    created_by_id: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = ["instrument_id", "object_name", "pointing_position", "date_range", "external_observation_id", "type", "status", "pointing_angle", "exposure_time", "reason", "description", "proposal_reference", "object_position", "depth", "bandpass", "t_resolution", "em_res_power", "o_ucd", "pol_states", "pol_xel", "category", "priority", "tracking_type", "created_on", "created_by_id"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ObservationCreate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of pointing_position
        if self.pointing_position:
            _dict['pointing_position'] = self.pointing_position.to_dict()
        # override the default output from pydantic by calling `to_dict()` of date_range
        if self.date_range:
            _dict['date_range'] = self.date_range.to_dict()
        # override the default output from pydantic by calling `to_dict()` of object_position
        if self.object_position:
            _dict['object_position'] = self.object_position.to_dict()
        # override the default output from pydantic by calling `to_dict()` of depth
        if self.depth:
            _dict['depth'] = self.depth.to_dict()
        # override the default output from pydantic by calling `to_dict()` of bandpass
        if self.bandpass:
            _dict['bandpass'] = self.bandpass.to_dict()
        # set to None if pointing_position (nullable) is None
        # and model_fields_set contains the field
        if self.pointing_position is None and "pointing_position" in self.model_fields_set:
            _dict['pointing_position'] = None

        # set to None if pointing_angle (nullable) is None
        # and model_fields_set contains the field
        if self.pointing_angle is None and "pointing_angle" in self.model_fields_set:
            _dict['pointing_angle'] = None

        # set to None if exposure_time (nullable) is None
        # and model_fields_set contains the field
        if self.exposure_time is None and "exposure_time" in self.model_fields_set:
            _dict['exposure_time'] = None

        # set to None if reason (nullable) is None
        # and model_fields_set contains the field
        if self.reason is None and "reason" in self.model_fields_set:
            _dict['reason'] = None

        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict['description'] = None

        # set to None if proposal_reference (nullable) is None
        # and model_fields_set contains the field
        if self.proposal_reference is None and "proposal_reference" in self.model_fields_set:
            _dict['proposal_reference'] = None

        # set to None if object_position (nullable) is None
        # and model_fields_set contains the field
        if self.object_position is None and "object_position" in self.model_fields_set:
            _dict['object_position'] = None

        # set to None if depth (nullable) is None
        # and model_fields_set contains the field
        if self.depth is None and "depth" in self.model_fields_set:
            _dict['depth'] = None

        # set to None if t_resolution (nullable) is None
        # and model_fields_set contains the field
        if self.t_resolution is None and "t_resolution" in self.model_fields_set:
            _dict['t_resolution'] = None

        # set to None if em_res_power (nullable) is None
        # and model_fields_set contains the field
        if self.em_res_power is None and "em_res_power" in self.model_fields_set:
            _dict['em_res_power'] = None

        # set to None if o_ucd (nullable) is None
        # and model_fields_set contains the field
        if self.o_ucd is None and "o_ucd" in self.model_fields_set:
            _dict['o_ucd'] = None

        # set to None if pol_states (nullable) is None
        # and model_fields_set contains the field
        if self.pol_states is None and "pol_states" in self.model_fields_set:
            _dict['pol_states'] = None

        # set to None if pol_xel (nullable) is None
        # and model_fields_set contains the field
        if self.pol_xel is None and "pol_xel" in self.model_fields_set:
            _dict['pol_xel'] = None

        # set to None if category (nullable) is None
        # and model_fields_set contains the field
        if self.category is None and "category" in self.model_fields_set:
            _dict['category'] = None

        # set to None if priority (nullable) is None
        # and model_fields_set contains the field
        if self.priority is None and "priority" in self.model_fields_set:
            _dict['priority'] = None

        # set to None if tracking_type (nullable) is None
        # and model_fields_set contains the field
        if self.tracking_type is None and "tracking_type" in self.model_fields_set:
            _dict['tracking_type'] = None

        # set to None if created_on (nullable) is None
        # and model_fields_set contains the field
        if self.created_on is None and "created_on" in self.model_fields_set:
            _dict['created_on'] = None

        # set to None if created_by_id (nullable) is None
        # and model_fields_set contains the field
        if self.created_by_id is None and "created_by_id" in self.model_fields_set:
            _dict['created_by_id'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ObservationCreate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "instrument_id": obj.get("instrument_id"),
            "object_name": obj.get("object_name"),
            "pointing_position": Coordinate.from_dict(obj["pointing_position"]) if obj.get("pointing_position") is not None else None,
            "date_range": DateRange.from_dict(obj["date_range"]) if obj.get("date_range") is not None else None,
            "external_observation_id": obj.get("external_observation_id"),
            "type": obj.get("type"),
            "status": obj.get("status"),
            "pointing_angle": obj.get("pointing_angle"),
            "exposure_time": obj.get("exposure_time"),
            "reason": obj.get("reason"),
            "description": obj.get("description"),
            "proposal_reference": obj.get("proposal_reference"),
            "object_position": Coordinate.from_dict(obj["object_position"]) if obj.get("object_position") is not None else None,
            "depth": UnitValue.from_dict(obj["depth"]) if obj.get("depth") is not None else None,
            "bandpass": Bandpass.from_dict(obj["bandpass"]) if obj.get("bandpass") is not None else None,
            "t_resolution": obj.get("t_resolution"),
            "em_res_power": obj.get("em_res_power"),
            "o_ucd": obj.get("o_ucd"),
            "pol_states": obj.get("pol_states"),
            "pol_xel": obj.get("pol_xel"),
            "category": obj.get("category"),
            "priority": obj.get("priority"),
            "tracking_type": obj.get("tracking_type"),
            "created_on": obj.get("created_on"),
            "created_by_id": obj.get("created_by_id")
        })
        return _obj





# coding: utf-8

"""
    ACROSS Server

    Server providing tools and utilities for various NASA missions to aid in coordination of large observation efforts.

    The version of the OpenAPI document: 1.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from across.sdk.v1.models.id_name_schema import IDNameSchema
from across.sdk.v1.models.nullable_date_range import NullableDateRange
from across.sdk.v1.models.observatory_ephemeris_type import ObservatoryEphemerisType
from across.sdk.v1.models.observatory_type import ObservatoryType
from typing import Optional, Set
from typing_extensions import Self

class Observatory(BaseModel):
    """
    A Pydantic model class representing a created observatory  Notes ----- Inherits from ObservatoryBase
    """ # noqa: E501
    id: StrictStr
    created_on: datetime
    name: StrictStr
    short_name: StrictStr
    type: ObservatoryType
    telescopes: Optional[List[IDNameSchema]] = None
    ephemeris_types: Optional[List[ObservatoryEphemerisType]] = None
    reference_url: Optional[StrictStr]
    operational: Optional[NullableDateRange]
    __properties: ClassVar[List[str]] = ["id", "created_on", "name", "short_name", "type", "telescopes", "ephemeris_types", "reference_url", "operational"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Observatory from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in telescopes (list)
        _items = []
        if self.telescopes:
            for _item_telescopes in self.telescopes:
                if _item_telescopes:
                    _items.append(_item_telescopes.to_dict())
            _dict['telescopes'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in ephemeris_types (list)
        _items = []
        if self.ephemeris_types:
            for _item_ephemeris_types in self.ephemeris_types:
                if _item_ephemeris_types:
                    _items.append(_item_ephemeris_types.to_dict())
            _dict['ephemeris_types'] = _items
        # override the default output from pydantic by calling `to_dict()` of operational
        if self.operational:
            _dict['operational'] = self.operational.to_dict()
        # set to None if telescopes (nullable) is None
        # and model_fields_set contains the field
        if self.telescopes is None and "telescopes" in self.model_fields_set:
            _dict['telescopes'] = None

        # set to None if ephemeris_types (nullable) is None
        # and model_fields_set contains the field
        if self.ephemeris_types is None and "ephemeris_types" in self.model_fields_set:
            _dict['ephemeris_types'] = None

        # set to None if reference_url (nullable) is None
        # and model_fields_set contains the field
        if self.reference_url is None and "reference_url" in self.model_fields_set:
            _dict['reference_url'] = None

        # set to None if operational (nullable) is None
        # and model_fields_set contains the field
        if self.operational is None and "operational" in self.model_fields_set:
            _dict['operational'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Observatory from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "created_on": obj.get("created_on"),
            "name": obj.get("name"),
            "short_name": obj.get("short_name"),
            "type": obj.get("type"),
            "telescopes": [IDNameSchema.from_dict(_item) for _item in obj["telescopes"]] if obj.get("telescopes") is not None else None,
            "ephemeris_types": [ObservatoryEphemerisType.from_dict(_item) for _item in obj["ephemeris_types"]] if obj.get("ephemeris_types") is not None else None,
            "reference_url": obj.get("reference_url"),
            "operational": NullableDateRange.from_dict(obj["operational"]) if obj.get("operational") is not None else None
        })
        return _obj


